       IDENTIFICATION DIVISION.
       PROGRAM-ID.    SIMODATE.
       AUTHOR.        SIMOTIME TECHNOLOGIES.
      *****************************************************************
      * Copyright (C) 1987-2019 SimoTime Technologies.                *
      *                                                               *
      * All rights reserved.  Unpublished, all rights reserved under  *
      * copyright law and international treaty.  Use of a copyright   *
      * notice is precautionary only and does not imply publication   *
      * or disclosure.                                                *
      *                                                               *
      * Permission to use, copy, modify and distribute this software  *
      * for any non-commercial purpose and without fee is hereby      *
      * granted, provided the SimoTime copyright notice appear on all *
      * copies of the software. The SimoTime name or Logo may not be  *
      * used in any advertising or publicity pertaining to the use    *
      * of the software without the written permission of SimoTime    *
      * Technologies.                                                 *
      *                                                               *
      * Permission to use, copy, modify and distribute this software  *
      * for any commercial purpose requires a fee to be paid to       *
      * SimoTime Technologies. Once the fee is received by SimoTime   *
      * the latest version of the software will be delivered and a    *
      * license will be granted for use within an enterprise,         *
      * provided the SimoTime copyright notice appear on all copies   *
      * of the software. The SimoTime name or Logo may not be used    *
      * in any advertising or publicity pertaining to the use of the  *
      * software without the written permission of SimoTime           *
      * Technologies.                                                 *
      *                                                               *
      * SimoTime Technologies makes no warranty or representations    *
      * about the suitability of the software for any purpose. It is  *
      * provided "AS IS" without any expressed or implied warranty,   *
      * including the implied warranties of merchantability, fitness  *
      * for a particular purpose and non-infringement. SimoTime       *
      * Technologies shall not be liable for any direct, indirect,    *
      * special or consequential damages resulting from the loss of   *
      * use, data or projects, whether in an action of contract or    *
      * tort, arising out of or in connection with the use or         *
      * performance of this software                                  *
      *                                                               *
      * SimoTime Technologies                                         *
      * 15 Carnoustie Drive                                           *
      * Novato, CA 94949-5849                                         *
      * 415.883.6565                                                  *
      *                                                               *
      * RESTRICTED RIGHTS LEGEND                                      *
      * Use, duplication, or disclosure by the Government is subject  *
      * to restrictions as set forth in subparagraph (c)(1)(ii) of    *
      * the Rights in Technical Data and Computer Software clause at  *
      * DFARS 52.227-7013 or subparagraphs (c)(1) and (2) of          *
      * Commercial  Computer Software - Restricted Rights  at 48      *
      * CFR 52.227-19, as applicable.  Contact SimoTime Technologies, *
      * 15 Carnoustie Drive, Novato, CA 94949-5849.                   *
      *                                                               *
      *****************************************************************
      *      This program is provided by SimoTime Technologies        *
      *        Our e-mail address is: helpdesk@simotime.com           *
      *     Also, visit our Web Site at http://www.simotime.com       *
      *                                                               *
      *****************************************************************
      *
      *****************************************************************
      * Source Member: SIMODATE.CBL
      * Copy Files:    PASSDATE.CPY
      *****************************************************************
      *
      * SIMODATE - edits and formats a gregorian date ccyymmdd. It also
      * provides additional information such as:
      *
      * 1. Identify leap year
      * 2. Provide a Julian Date
      * 3. Provide a text date - for example, January 1, 2001
      * 4. Provide days-in-month
      * and more...
      *
      * CALLING PROTOCOL
      * ----------------
      *    call 'SIMODATE' using STD-SIMODATE
      *
      *    01  STD-SIMODATE.
      *        05  STD-REQUEST             pic X(8).
      *        05  STD-RESPONSE            pic 9(4).
      *        05  STD-MESSAGE-TEXT        pic X(68).
      *        05  STD-GREGORIAN-4-EDIT    pic X(10).
      *        05  STD-EDITED-INFO.
      *            10  STD-DEBUG-INFO      pic X(8).
      *            10  STD-LEAP-YEAR-YN    pic X.
      *            10  STD-MONTH-VERBAGE   pic X(10).
      *            10  STD-MM-DAYS         pic 99.
      *            10  STD-GREGORIAN-DATE  pic 9(8).
      *            10  STD-JULIAN-DATE     pic 9(7).
      *            10  STD-JULIAN-VALUE    redefines STD-JULIAN-DATE.
      *                15  STD-JULIAN-CCYY pic 9(4).
      *                15  STD-JULIAN-DAY  pic 9(3).
      *            10  STD-DAYS-REMAINING  pic 9(3).
      *            10  STD-DATE-VERBAGE    pic X(18).
      *            10  STD-DATE-EDIT-BYTE  pic X.
      *            10  STD-DATE-EDITED     pic X(10).
      *
      *****************************************************************
      *
      * MAINTENANCE
      * -----------
      * 1997/02/27 Simmons, Created program.
      * 2004/01/26 Simmons, Mike added the check for 4000th year
      *                     increments are not leap years.
      *
      *****************************************************************
      *
       DATA DIVISION.
       WORKING-STORAGE SECTION.
      *
      *****************************************************************
      *    Data-structure for Title and Copyright...
      *****************************************************************
       01  SIM-TITLE.
           05  T1 pic X(11) value '* SIMODATE '.
           05  T2 pic X(34) value 'Process a Date Request            '.
           05  T3 pic X(10) value ' v13.06.19'.
           05  T4 pic X(24) value ' http://www.simotime.com'.
       01  SIM-COPYRIGHT.
           05  C1 pic X(11) value '* SIMODATE '.
           05  C2 pic X(20) value 'Copyright 1987-2019 '.
           05  C3 pic X(28) value '   SimoTime Technologies    '.
           05  C4 pic X(20) value ' All Rights Reserved'.

      *****************************************************************
       01  FIRST-TIME              pic X       value 'Y'.

       01  IX-1                    pic 999     value 0.
       01  IX-2                    pic 999     value 0.
       01  WORK-N-7                pic 9(7)    value 0.

       01  DATE-WORK-ALPHA.
           05  DATE-WORK-A-CCYY    pic X(4)    value SPACES.
           05  DATE-WORK-A-MM      pic X(2)    value SPACES.
           05  DATE-WORK-A-DD      pic X(2)    value SPACES.

       01  DATE-WORK-NUMERIC.
           05  DATE-WORK-N-CCYY    pic 9(4)    value 0.
           05  DATE-WORK-N-MM      pic 9(2)    value 0.
           05  DATE-WORK-N-DD      pic 9(2)    value 0.

       01  MONTH-TABLE-DATA.
           05  filler              pic X(10)   value 'January   '.
           05  filler              pic X(10)   value 'February  '.
           05  filler              pic X(10)   value 'March     '.
           05  filler              pic X(10)   value 'April     '.
           05  filler              pic X(10)   value 'May       '.
           05  filler              pic X(10)   value 'June      '.
           05  filler              pic X(10)   value 'July      '.
           05  filler              pic X(10)   value 'August    '.
           05  filler              pic X(10)   value 'September '.
           05  filler              pic X(10)   value 'October   '.
           05  filler              pic X(10)   value 'November  '.
           05  filler              pic X(10)   value 'December  '.
           05  filler              pic 99      value 31.
           05  filler              pic 99      value 28.
           05  filler              pic 99      value 31.
           05  filler              pic 99      value 30.
           05  filler              pic 99      value 31.
           05  filler              pic 99      value 30.
           05  filler              pic 99      value 31.
           05  filler              pic 99      value 31.
           05  filler              pic 99      value 30.
           05  filler              pic 99      value 31.
           05  filler              pic 99      value 30.
           05  filler              pic 99      value 31.
       01  MONTH-TABLE             redefines MONTH-TABLE-DATA.
           05  MONTH-VERBAGE       pic X(10)   occurs 12 times.
           05  MONTH-COUNT-OF-DAYS pic 99      occurs 12 times.

       01  JULIAN-DATE.
           05  JULIAN-CCYY         pic 9(4)    value 0.
           05  JULIAN-DDD          pic 9(3)    value 0.

       01  DATE-VERBAGE            pic X(18)   value SPACES.

       01  DAY-OF-WEEK-DATA.
           05  filler              pic X(10)   value 'Sunday    '.
           05  filler              pic X(10)   value 'Monday    '.
           05  filler              pic X(10)   value 'Tuesday   '.
           05  filler              pic X(10)   value 'Wednesday '.
           05  filler              pic X(10)   value 'Thursday  '.
           05  filler              pic X(10)   value 'Friday    '.
           05  filler              pic X(10)   value 'Saturday  '.
       01  DOW-TABLE               redefines DAY-OF-WEEK-DATA.
           05  DOW-VERBAGE         pic X(10)   occurs 7 times.
       01  DOW-IDX                 pic 9       value 0.

      *****************************************************************
       01  MESSAGE-BUFFER.
           05  MESSAGE-HEADER      pic X(11)   value '* SIMODATE '.
           05  MESSAGE-TEXT        pic X(68).

       01  DIFF-OF-YEARS           pic 9(4)    value 0.
       01  DIFF-OF-DAYS            pic 9(7)    value 0.
       01  DIFF-OF-WEEKS           pic 9(7)    value 0.
       01  MULTIPLE-YEARS          pic 9(7)    value 0.
       01  NUMBER-OF-LEAP-YEARS    pic 9(7)    value 0.
       01  HOLD-REMAINDER-1        pic 9(7)    value 0.
       01  HOLD-REMAINDER-2        pic 9(7)    value 0.
       01  REGISTER-7-0            pic 9(7)    value 0.

       COPY PASSDIFF.

      *****************************************************************
       LINKAGE SECTION.
       COPY PASSDATE.

      *****************************************************************
      * Mainline processing routine...
      *****************************************************************
       PROCEDURE DIVISION using STD-SIMODATE.

           add 20 to ZERO giving STD-RESPONSE
           move SPACES to STD-MESSAGE-TEXT

           if  FIRST-TIME not = 'N'
               if  STD-DEBUG-INFO = 'DEBUG   '
               or  STD-DEBUG-INFO = 'TRACE   '
                   perform Z-POST-COPYRIGHT
               end-if
               move 'N' to FIRST-TIME
           end-if

           evaluate STD-REQUEST
               when 'EDITDATE' perform EDIT-DATE-FUNCTION
               when 'EDITGDAT' perform ACTION-GDAT-REQUEST
               when OTHER      add 16 to ZERO giving RETURN-CODE
                               move '0016' to STD-RESPONSE
           end-evaluate

           if  STD-RESPONSE = 20
           and STD-MESSAGE-TEXT = SPACES
               subtract STD-RESPONSE from STD-RESPONSE
           end-if

           GOBACK.

      *****************************************************************
      *    This routine will accept dates in the following formats
      *     ccyymmdd
      *      ccyy/mm/dd
      *      ccyy-mm-dd
      *      ccyy.mm.dd
      *****************************************************************
       ACTION-GDAT-REQUEST.
           move SPACES to DATE-WORK-ALPHA
           add 1 to ZERO giving IX-2
           move STD-GREGORIAN-4-EDIT to DATE-WORK-A-CCYY
           if  DATE-WORK-A-CCYY not NUMERIC
               move 'NOK for Date Value, CCYY not numeric value...'
                 to MESSAGE-TEXT
               perform Z-POST-ABEND
           else
               add 4 to IX-2
           end-if
      *    --------------------------------------------------------
      *    Possible non-numeric separator between CCYY and MM
           if  STD-GREGORIAN-4-EDIT(IX-2:1) not NUMERIC
               add 1 to IX-2
           end-if
           move STD-GREGORIAN-4-EDIT(IX-2:2) to DATE-WORK-A-MM
           if  DATE-WORK-A-MM not NUMERIC
               move DATE-WORK-A-MM(1:1) to DATE-WORK-A-MM(2:1)
               move ZERO                to DATE-WORK-A-MM(1:1)
               add 1 to IX-2
           else
               add 2 to IX-2
           end-if
      *    --------------------------------------------------------
      *    Possible non-numeric separator between MM and DD
           if  STD-GREGORIAN-4-EDIT(IX-2:1) not NUMERIC
               add 1 to IX-2
           end-if
           move STD-GREGORIAN-4-EDIT(IX-2:2) to DATE-WORK-A-DD
           if  DATE-WORK-A-DD not NUMERIC
               move DATE-WORK-A-DD(1:1) to DATE-WORK-A-DD(2:1)
               move ZERO                to DATE-WORK-A-DD(1:1)
           end-if
           if  DATE-WORK-A-MM not NUMERIC
               move 'NOK for Date Value, MM not numeric value...'
                 to MESSAGE-TEXT
               perform Z-POST-ABEND
           end-if
           if  DATE-WORK-A-DD not NUMERIC
               move 'NOK for Date Value, DD not numeric value...'
                 to MESSAGE-TEXT
               perform Z-POST-ABEND
           end-if
           move 'WIP for Date Function...' to MESSAGE-TEXT
           perform Z-POST-CONDITIONAL-TRACE
           move DATE-WORK-ALPHA to DATE-WORK-NUMERIC
           perform FUNCTION-LEAP-YEAR.

           perform FUNCTION-FOR-MONTH
           perform FUNCTION-FOR-DAY
           perform FUNCTION-CREATE-TXT-DATE
           perform FUNCTION-CREATE-NBR-DATE
           perform FUNCTION-CONVERT-GDATE-2-JDATE
           perform FUNCTION-CALCULATE-DAYS-LEFT

           perform FUNCTION-DAY-OF-WEEK

      *    ------------------------------------------------------------
      *    Determine AOK for Date Message...
           move 'AOK for Date Function...' to MESSAGE-TEXT
           perform Z-POST-CONDITIONAL-TRACE

           exit.

      *****************************************************************
      *>   This routine will accept dates in the following formats
      *>     ccyymmdd
      *>     ccyy/mm/dd
      *>     ccyy-mm-dd
      *>     ccyy.mm.dd
      *****************************************************************
       EDIT-DATE-FUNCTION.
           move SPACES to DATE-WORK-ALPHA
           add 1 to ZERO giving IX-2
           move STD-GREGORIAN-4-EDIT to DATE-WORK-A-CCYY
           if  DATE-WORK-A-CCYY not NUMERIC
               move 'NOK for Date Value, CCYY not numeric value...'
                 to MESSAGE-TEXT
               perform Z-POST-ABEND
           else
               add 4 to IX-2
           end-if
      *>   --------------------------------------------------------
      *>   Possible non-numeric separator between CCYY and MM
           if  STD-GREGORIAN-4-EDIT(IX-2:1) not NUMERIC
               add 1 to IX-2
           end-if
           move STD-GREGORIAN-4-EDIT(IX-2:2) to DATE-WORK-A-MM
           if  DATE-WORK-A-MM not NUMERIC
               move DATE-WORK-A-MM(1:1) to DATE-WORK-A-MM(2:1)
               move ZERO                to DATE-WORK-A-MM(1:1)
               add 1 to IX-2
           else
               add 2 to IX-2
           end-if
      *>   --------------------------------------------------------
      *>   Possible non-numeric separator between MM and DD
           if  STD-GREGORIAN-4-EDIT(IX-2:1) not NUMERIC
               add 1 to IX-2
           end-if
           move STD-GREGORIAN-4-EDIT(IX-2:2) to DATE-WORK-A-DD
           if  DATE-WORK-A-DD not NUMERIC
               move DATE-WORK-A-DD(1:1) to DATE-WORK-A-DD(2:1)
               move ZERO                to DATE-WORK-A-DD(1:1)
           end-if
           if  DATE-WORK-A-MM not NUMERIC
               move 'NOK for Date Value, MM not numeric value...'
                 to MESSAGE-TEXT
               perform Z-POST-ABEND
           end-if
           if  DATE-WORK-A-DD not NUMERIC
               move 'NOK for Date Value, DD not numeric value...'
                 to MESSAGE-TEXT
               perform Z-POST-ABEND
           end-if
           move 'WIP for Date Function...' to MESSAGE-TEXT
           perform Z-POST-CONDITIONAL-TRACE
           move DATE-WORK-ALPHA to DATE-WORK-NUMERIC
      *   ------------------------------------------------------------
      *   Determine if a leap year, it is not true that every fourth
      *   year is a leap year.
      *
      *   if CCYY is divisible by 4 it is a  leap year
      *      except
      *      if divisible by 100 then it is not a leap year
      *         except
      *         if divisible by 400 then it is a leap year
      *            except
      *            if divisible by 4000 then it is not a leap year.
      *
      *    1600,2000, etc... are leap years
      *    1700,1800,1900 are not leap years
      *    4000,8000,etc... are not leap years
      *
      *    In other words the 100th year increments are not leap
      *    years with the exception that every 400th year increment
      *    is a leap year except every 4000th year is not a leap year.
      *
           move 'N' to STD-LEAP-YEAR-YN
           add 28 to ZERO giving MONTH-COUNT-OF-DAYS(2)
           move 'WIP for Date, not a Leap Year...' to MESSAGE-TEXT
           divide DATE-WORK-N-CCYY by 4 giving WORK-N-7
           multiply 4 by WORK-N-7
           if  WORK-N-7 = DATE-WORK-N-CCYY
               divide DATE-WORK-N-CCYY by 100 giving WORK-N-7
               multiply 100 by WORK-N-7
               if  WORK-N-7  = DATE-WORK-N-CCYY
                   divide DATE-WORK-N-CCYY by 400 giving WORK-N-7
                   multiply 400 by WORK-N-7
                   if  WORK-N-7 = DATE-WORK-N-CCYY
                       divide DATE-WORK-N-CCYY by 4000 giving WORK-N-7
                       multiply 4000 by WORK-N-7
                       if  WORK-N-7 not = DATE-WORK-N-CCYY
                           move 'Y' to STD-LEAP-YEAR-YN
                       end-if
                   end-if
               else
                   move 'Y' to STD-LEAP-YEAR-YN
               end-if
               if  STD-LEAP-YEAR-YN = 'Y'
                   move 'Y' to STD-LEAP-YEAR-YN
                   add 29 to ZERO giving MONTH-COUNT-OF-DAYS(2)
                   move 'WIP for Date, it is a Leap Year...'
                     to MESSAGE-TEXT
               end-if
           end-if
           perform Z-POST-CONDITIONAL-TRACE
      *>   ------------------------------------------------------------
      *>   Test for a valid month...
           if  DATE-WORK-N-MM > 0
           and DATE-WORK-N-MM < 13
               move 'WIP for Date, month is ?????????' to MESSAGE-TEXT
               move MONTH-VERBAGE(DATE-WORK-N-MM)
                 to MESSAGE-TEXT(24:10)
               perform Z-POST-CONDITIONAL-TRACE
               move MONTH-VERBAGE(DATE-WORK-N-MM)
                 to MESSAGE-TEXT(24:10)
               move MONTH-VERBAGE(DATE-WORK-N-MM) to STD-MONTH-VERBAGE
           else
               move 'NOK for Date Value, MM not 1 through 12...'
                 to MESSAGE-TEXT
               perform Z-POST-ABEND

           end-if
      *>   ------------------------------------------------------------
      *>   Test for a valid day...
           if  DATE-WORK-N-DD > 0
           and DATE-WORK-N-DD < MONTH-COUNT-OF-DAYS(DATE-WORK-N-MM) + 1
               move 'WIP for Date, month has ?? days' to MESSAGE-TEXT
               move MONTH-COUNT-OF-DAYS(DATE-WORK-N-MM)
                 to MESSAGE-TEXT(25:2)
               perform Z-POST-CONDITIONAL-TRACE
           else
               move 'NOK for Day Value, DD not within month range...'
                 to MESSAGE-TEXT
               perform Z-POST-ABEND
           end-if
           move MONTH-COUNT-OF-DAYS(DATE-WORK-N-MM)
             to STD-MM-DAYS

      *>   ------------------------------------------------------------
      *>   Create text version of the date. This is a sixteen character
      *>   field (DATE-VERBAGE) that is left-justified in the following
      *>   format...      month dd, ccyy
           move 'WIP for Date, ' to MESSAGE-TEXT
           move MONTH-VERBAGE(DATE-WORK-N-MM) to DATE-VERBAGE
           subtract IX-2 from IX-2
           inspect DATE-VERBAGE tallying IX-2
                   for CHARACTERS before SPACE
           add 2 to IX-2
           if  DATE-WORK-A-DD(1:1) = '0'
               move DATE-WORK-A-DD(2:1) to DATE-VERBAGE(IX-2:1)
               add 1 to IX-2
           else
               move DATE-WORK-A-DD to DATE-VERBAGE(IX-2:2)
               add 2 to IX-2
           end-if
           move ', ' to DATE-VERBAGE(IX-2:2)
           add 2 to IX-2
           move DATE-WORK-N-CCYY to DATE-VERBAGE(IX-2:4)

           move DATE-VERBAGE to MESSAGE-TEXT(15:18)
           perform Z-POST-CONDITIONAL-TRACE

           move DATE-VERBAGE to STD-DATE-VERBAGE

      *>   ------------------------------------------------------------
      *>   Create numeric date for display or print...
           move DATE-WORK-A-CCYY   to STD-DATE-EDITED
           move STD-DATE-EDIT-BYTE to STD-DATE-EDITED(5:1)
           move DATE-WORK-A-MM     to STD-DATE-EDITED(6:2)
           move STD-DATE-EDIT-BYTE to STD-DATE-EDITED(8:1)
           move DATE-WORK-A-DD     to STD-DATE-EDITED(9:2)
           move DATE-WORK-NUMERIC  to STD-GREGORIAN-DATE

      *>   ------------------------------------------------------------
      *>   Determine Julian Date...
           move 'WIP for Date, Julian is CCYYDDD' to MESSAGE-TEXT
           move DATE-WORK-N-CCYY to JULIAN-CCYY
           add DATE-WORK-N-DD to ZERO giving JULIAN-DDD
           add 1 to ZERO giving IX-2
           perform until IX-2 = DATE-WORK-N-MM
                      or IX-2 > 12
               add MONTH-COUNT-OF-DAYS(IX-2) to JULIAN-DDD
               add 1 to IX-2
           end-perform
           move JULIAN-DATE to MESSAGE-TEXT(25:7)
           perform Z-POST-CONDITIONAL-TRACE
           move JULIAN-DATE to STD-JULIAN-DATE

      *>   ------------------------------------------------------------
      *>   Calculate remaining days in the year...
           if  STD-LEAP-YEAR-YN = 'Y'
               subtract STD-JULIAN-DAY from 366
                 giving STD-DAYS-REMAINING
           else
               subtract STD-JULIAN-DAY from 365
                 giving STD-DAYS-REMAINING
           end-if

      *>   ------------------------------------------------------------
      *>   Determine AOK for Date Message...
           move 'AOK for Date Function...' to MESSAGE-TEXT
           perform Z-POST-CONDITIONAL-TRACE
           exit.

      *****************************************************************
       FUNCTION-CREATE-TXT-DATE.
      *    ------------------------------------------------------------
      *    Create text version of the date. This is a sixteen character
      *    field (DATE-VERBAGE) that is left-justified in the following
      *    format...      month dd, ccyy
           move 'WIP for Date, ' to MESSAGE-TEXT
           move MONTH-VERBAGE(DATE-WORK-N-MM) to DATE-VERBAGE
           subtract IX-2 from IX-2
           inspect DATE-VERBAGE tallying IX-2
                   for CHARACTERS before SPACE
           add 2 to IX-2
           if  DATE-WORK-A-DD(1:1) = '0'
               move DATE-WORK-A-DD(2:1) to DATE-VERBAGE(IX-2:1)
               add 1 to IX-2
           else
               move DATE-WORK-A-DD to DATE-VERBAGE(IX-2:2)
               add 2 to IX-2
           end-if
           move ', ' to DATE-VERBAGE(IX-2:2)
           add 2 to IX-2
           move DATE-WORK-N-CCYY to DATE-VERBAGE(IX-2:4)

           move DATE-VERBAGE to MESSAGE-TEXT(15:18)
           perform Z-POST-CONDITIONAL-TRACE

           move DATE-VERBAGE to STD-DATE-VERBAGE
           exit.

      *****************************************************************
       FUNCTION-DAY-OF-WEEK.
      *    Determine the Day of a week as a number and text.
      *   ----------------------------------------------------------- *
           move ZERO to STD-DAY-OF-WEEK-NBR
           move 'Unknown   ' to STD-DAY-OF-WEEK-TXT
           add 1600001 to ZERO giving DDF-JULIAN-DATE-1
           move STD-JULIAN-DATE to DDF-JULIAN-DATE-2
           perform FUNCTION-DATE-DIFFERENCE
           divide DDF-DIFFERENCE-IN-DAYS by 7
           giving DIFF-OF-WEEKS remainder DOW-IDX
           evaluate DOW-IDX
             when 0     add 6 to ZERO giving STD-DAY-OF-WEEK-NBR
             when 1     add 7 to ZERO giving STD-DAY-OF-WEEK-NBR
             when 2     add 1 to ZERO giving STD-DAY-OF-WEEK-NBR
             when 3     add 2 to ZERO giving STD-DAY-OF-WEEK-NBR
             when 4     add 3 to ZERO giving STD-DAY-OF-WEEK-NBR
             when 5     add 4 to ZERO giving STD-DAY-OF-WEEK-NBR
             when 6     add 5 to ZERO giving STD-DAY-OF-WEEK-NBR
           end-evaluate
           if  STD-DAY-OF-WEEK-NBR > 0
           and STD-DAY-OF-WEEK-NBR < 8
               move DOW-VERBAGE(STD-DAY-OF-WEEK-NBR)
                 to STD-DAY-OF-WEEK-TXT
           else
               move ZERO to STD-DAY-OF-WEEK-NBR
               move 'Unknown   ' to STD-DAY-OF-WEEK-TXT
           end-if
           exit.

      *****************************************************************
       FUNCTION-LEAP-YEAR.
      *   ----------------------------------------------------------- *
      *   Determine if a leap year, it is not true that every fourth
      *   year is a leap year.
      *
      *   if CCYY is divisible by 4 it is a  leap year
      *      except
      *      if divisible by 100 then it is not a leap year
      *         except
      *         if divisible by 400 then it is a leap year
      *            except
      *            if divisible by 4000 then it is not a leap year.
      *
      *    1600,2000, etc... are leap years
      *    1700,1800,1900 are not leap years
      *    4000,8000,etc... are not leap years
      *
      *    In other words the 100th year increments are not leap
      *    years with the exception that every 400th year increment
      *    is a leap year except every 4000th year is not a leap year.
      *
           move 'N' to STD-LEAP-YEAR-YN
           add 28 to ZERO giving MONTH-COUNT-OF-DAYS(2)
           move 'WIP for Date, not a Leap Year...' to MESSAGE-TEXT
           divide DATE-WORK-N-CCYY by 4 giving WORK-N-7
           multiply 4 by WORK-N-7
           if  WORK-N-7 = DATE-WORK-N-CCYY
               divide DATE-WORK-N-CCYY by 100 giving WORK-N-7
               multiply 100 by WORK-N-7
               if  WORK-N-7  = DATE-WORK-N-CCYY
                   divide DATE-WORK-N-CCYY by 400 giving WORK-N-7
                   multiply 400 by WORK-N-7
                   if  WORK-N-7 = DATE-WORK-N-CCYY
                       divide DATE-WORK-N-CCYY by 4000 giving WORK-N-7
                       multiply 4000 by WORK-N-7
                       if  WORK-N-7 not = DATE-WORK-N-CCYY
                           move 'Y' to STD-LEAP-YEAR-YN
                       end-if
                   end-if
               else
                   move 'Y' to STD-LEAP-YEAR-YN
               end-if
               if  STD-LEAP-YEAR-YN = 'Y'
                   move 'Y' to STD-LEAP-YEAR-YN
                   add 29 to ZERO giving MONTH-COUNT-OF-DAYS(2)
                   move 'WIP for Date, it is a Leap Year...'
                     to MESSAGE-TEXT
               end-if
           end-if
           perform Z-POST-CONDITIONAL-TRACE

           exit.

      *****************************************************************
       FUNCTION-FOR-MONTH.
      *    ------------------------------------------------------------
      *    Test for a valid month...
           if  DATE-WORK-N-MM > 0
           and DATE-WORK-N-MM < 13
               move 'WIP for Date, month is ?????????' to MESSAGE-TEXT
               move MONTH-VERBAGE(DATE-WORK-N-MM)
                 to MESSAGE-TEXT(24:10)
               perform Z-POST-CONDITIONAL-TRACE
               move MONTH-VERBAGE(DATE-WORK-N-MM)
                 to MESSAGE-TEXT(24:10)
               move MONTH-VERBAGE(DATE-WORK-N-MM) to STD-MONTH-VERBAGE
           else
               move 'NOK for Date Value, MM not 1 through 12...'
                 to MESSAGE-TEXT
               perform Z-POST-ABEND

           end-if
           exit.

      *****************************************************************
       FUNCTION-FOR-DAY.
      *    ------------------------------------------------------------
      *    Test for a valid day...
           if  DATE-WORK-N-DD > 0
           and DATE-WORK-N-DD < MONTH-COUNT-OF-DAYS(DATE-WORK-N-MM) + 1
               move 'WIP for Date, month has ?? days' to MESSAGE-TEXT
               move MONTH-COUNT-OF-DAYS(DATE-WORK-N-MM)
                 to MESSAGE-TEXT(25:2)
               perform Z-POST-CONDITIONAL-TRACE
           else
               move 'NOK for Day Value, DD not within month range...'
                 to MESSAGE-TEXT
               perform Z-POST-ABEND
           end-if
           move MONTH-COUNT-OF-DAYS(DATE-WORK-N-MM)
             to STD-MM-DAYS
           exit.

      *****************************************************************
       FUNCTION-CREATE-NBR-DATE.
      *    ------------------------------------------------------------
      *    Create numeric date for display or print...
           move DATE-WORK-A-CCYY   to STD-DATE-EDITED
           move STD-DATE-EDIT-BYTE to STD-DATE-EDITED(5:1)
           move DATE-WORK-A-MM     to STD-DATE-EDITED(6:2)
           move STD-DATE-EDIT-BYTE to STD-DATE-EDITED(8:1)
           move DATE-WORK-A-DD     to STD-DATE-EDITED(9:2)
           move DATE-WORK-NUMERIC  to STD-GREGORIAN-DATE
           exit.

      *****************************************************************
       FUNCTION-CONVERT-GDATE-2-JDATE.
      *    ------------------------------------------------------------
      *    Determine Julian Date...
           move 'WIP for Date, Julian is CCYYDDD' to MESSAGE-TEXT
           move DATE-WORK-N-CCYY to JULIAN-CCYY
           add DATE-WORK-N-DD to ZERO giving JULIAN-DDD
           add 1 to ZERO giving IX-2
           perform until IX-2 = DATE-WORK-N-MM
                      or IX-2 > 12
               add MONTH-COUNT-OF-DAYS(IX-2) to JULIAN-DDD
               add 1 to IX-2
           end-perform
           move JULIAN-DATE to MESSAGE-TEXT(25:7)
           perform Z-POST-CONDITIONAL-TRACE
           move JULIAN-DATE to STD-JULIAN-DATE
           exit.

      *****************************************************************
       FUNCTION-CALCULATE-DAYS-LEFT.
      *    ------------------------------------------------------------
      *    Calculate remaining days in the year...
           if  STD-LEAP-YEAR-YN = 'Y'
               subtract STD-JULIAN-DAY from 366
                 giving STD-DAYS-REMAINING
           else
               subtract STD-JULIAN-DAY from 365
                 giving STD-DAYS-REMAINING
           end-if
           exit.



      *****************************************************************
       FUNCTION-DATE-DIFFERENCE.
      *    Calculate difference of years...
           subtract DDF-JULIAN-CCYY-1 from DDF-JULIAN-CCYY-2
             giving DIFF-OF-YEARS
      *    Calculate difference of days...
           subtract DIFF-OF-DAYS from DIFF-OF-DAYS
           if  DIFF-OF-YEARS = 0
               subtract DDF-JULIAN-DAY-1 from DDF-JULIAN-DAY-2
                 giving DIFF-OF-DAYS
           else
               add DDF-DAYS-REMAINING-1 to DIFF-OF-DAYS
               add DDF-JULIAN-DAY-2     to DIFF-OF-DAYS
           end-if
      *    Calculate for multiple years ignoring leap years
           if  DIFF-OF-YEARS > 1
               subtract 1 from DIFF-OF-YEARS giving MULTIPLE-YEARS
               multiply 365 by MULTIPLE-YEARS giving REGISTER-7-0
               add REGISTER-7-0 to DIFF-OF-DAYS
           end-if
      *    Adjust difference of days for leap-year cycle...
           if  DIFF-OF-YEARS > 1
               perform FUNCTION-DATE-DIFF-LEAP
               add NUMBER-OF-LEAP-YEARS to DIFF-OF-DAYS
           end-if
           add DIFF-OF-DAYS to ZERO giving DDF-DIFFERENCE-IN-DAYS
           exit.

      *****************************************************************
       FUNCTION-DATE-DIFF-LEAP.
      *    Calculate years to next leap year...
           divide DDF-JULIAN-CCYY-1 by 4 giving MULTIPLE-YEARS
                  remainder HOLD-REMAINDER-1
      *    Calculate number of years between primary and secondary...
           subtract 1 from DIFF-OF-YEARS giving MULTIPLE-YEARS
           add HOLD-REMAINDER-1 to MULTIPLE-YEARS
           divide MULTIPLE-YEARS by 4 giving NUMBER-OF-LEAP-YEARS
      *    Adjust difference of days for 400-year leap-year cycle...
           perform FUNCTION-DATE-DIFF-LEAP-400
           exit.

      *****************************************************************
       FUNCTION-DATE-DIFF-LEAP-400.
      *    Calculate years to next leap year 400 exception...
           divide DDF-JULIAN-CCYY-1 by 400 giving MULTIPLE-YEARS
                  remainder HOLD-REMAINDER-2
      *    Calculate number of years between primary and secondary...
           subtract 1 from DIFF-OF-YEARS giving MULTIPLE-YEARS
           add HOLD-REMAINDER-2 to MULTIPLE-YEARS
           divide MULTIPLE-YEARS by 400 giving REGISTER-7-0
           subtract REGISTER-7-0 from NUMBER-OF-LEAP-YEARS
      *    Adjust difference of days for 4-year leap-year cycle...
           exit.




      *****************************************************************
      *    Display Copyright or Program Message...
      *    ------------------------------------------------------------
       Z-POST-ABEND.
           if  MESSAGE-TEXT not = SPACES
               move MESSAGE-TEXT to STD-MESSAGE-TEXT
               perform Z-POST-MESSAGE
           end-if
           move 'Program is ABENDING...' to MESSAGE-TEXT
           perform Z-POST-MESSAGE
           perform Z-POST-CONSOLE
           add 16 to ZERO giving STD-RESPONSE
           GOBACK.

       Z-POST-COPYRIGHT.
           display SIM-TITLE     upon console
           display SIM-COPYRIGHT upon console
           exit.

       Z-POST-CONSOLE.
           display MESSAGE-BUFFER upon console
           exit.

       Z-POST-MESSAGE.
           display MESSAGE-BUFFER upon console
           move SPACES to MESSAGE-TEXT
           exit.

       Z-POST-CONDITIONAL-TRACE.
           if  STD-DEBUG-INFO = 'DEBUG   '
           or  STD-DEBUG-INFO = 'TRACE   '
               display MESSAGE-BUFFER upon console
           end-if
           move SPACES to MESSAGE-TEXT
           exit.
      *****************************************************************
      *      This example is provided by SimoTime Technologies        *
      *        Our e-mail address is: helpdesk@simotime.com           *
      *     Also, visit our Web Site at http://www.simotime.com       *
      *****************************************************************
